@@ -12,7 +12,8 @@ restinio::request_handling_status_t handler( const restinio::request_handle_t& r
 			.append_header( restinio::http_field::server, "RESTinio hello world server" )
 			.append_header_date_field()
 			.append_header( restinio::http_field::content_type, "text/plain; charset=utf-8" )
-			.set_body( fmt::format( "{}: Hello world!", req->remote_endpoint() ) )
+			.set_body( fmt::format( "{}: Hello world!",
+					restinio::fmtlib_tools::streamed( req->remote_endpoint() ) ) )
 			.done();
 
 		return restinio::request_accepted();
