@@ -11,13 +11,17 @@ MxxRu::Cpp::exe_target {
 	cpp_source( "media-type.cpp" )
 	cpp_source( "content-type.cpp" )
 	cpp_source( "cache-control.cpp" )
+	cpp_source( "connection.cpp" )
 	cpp_source( "content-encoding.cpp" )
 	cpp_source( "accept.cpp" )
 	cpp_source( "accept-charset.cpp" )
 	cpp_source( "accept-encoding.cpp" )
 	cpp_source( "accept-language.cpp" )
+	cpp_source( "authorization.cpp" )
 	cpp_source( "content-disposition.cpp" )
 	cpp_source( "range.cpp" )
 	cpp_source( "user-agent.cpp" )
+	cpp_source( "transfer-encoding.cpp" )
+	cpp_source( "host.cpp" )
 }
 
